/**
 * Copyright (C) 2011 - 101loops.com <dev@101loops.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.crashnote.servlet.config;

import com.crashnote.logger.config.LoggerConfigFactory;

import javax.servlet.FilterConfig;
import java.util.*;

public class ServletConfigFactory<C extends ServletConfig>
        extends LoggerConfigFactory<C> {

    // SETUP ======================================================================================

    public ServletConfigFactory(final FilterConfig filterConfig, final C internalConfig) {
        super(internalConfig);

        // gather properties from filter config and apply them
        final Properties props = new Properties();
        final Enumeration params = filterConfig.getInitParameterNames();
        while (params.hasMoreElements()) {
            final String name = (String) params.nextElement();
            final String value = filterConfig.getInitParameter(name);
            props.setProperty(name, value);
        }
        applyProperties(props, false);
    }

    public ServletConfigFactory(final FilterConfig filterConfig) {
        this(filterConfig, (C) new ServletConfig());
    }

    // SHARED =====================================================================================

    @Override
    protected void applyProperties(final Properties props, final boolean strict) {
        super.applyProperties(props, strict);

        final String skipReqHeader = getProperty(props, ServletConfig.PROP_REP_HEADER_SKIP, strict);
        if (skipReqHeader != null) config.setSkipHeaderData(skipReqHeader);

        final String skipReqSession = getProperty(props, ServletConfig.PROP_REP_SESSION_SKIP, strict);
        if (skipReqSession != null) config.setSkipSessionData(skipReqSession);
    }

}
